/*
 *  libfb - FreeBASIC's runtime library
 *	Copyright (C) 2004-2005 Andre V. T. Vicentini (av1ctor@yahoo.com.br) and others.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * hook_locate_ex.c -- locate entrypoint, default to console mode
 *
 * chng: nov/2004 written [v1ctor]
 *
 */

#include "fb.h"
#include "fb_rterr.h"

/*:::::*/
FBCALL int fb_LocateEx( int row, int col, int cursor, int *current_pos )
{
    int tmp_current_pos = 0;
    int res = fb_ErrorSetNum( FB_RTERROR_OK );
    int start_y, end_y, con_width;

    fb_ConsoleGetView(&start_y, &end_y);
    fb_GetSize( &con_width, NULL );

    if( row!=0 && (row<start_y || row>end_y) ) {
        res = fb_ErrorSetNum( FB_RTERROR_ILLEGALFUNCTIONCALL );
    } else if( col!=0 && (col<1 || col>con_width) ) {
        res = fb_ErrorSetNum( FB_RTERROR_ILLEGALFUNCTIONCALL );
    } else {
        fb_DevScrnInit_NoOpen( );

        FB_LOCK();

        if( fb_hooks.locateproc ) {
            tmp_current_pos = fb_hooks.locateproc( row, col, cursor );
        } else {
            tmp_current_pos = fb_ConsoleLocate( row, col, cursor );
        }

        if( col!=0 )
            FB_HANDLE_SCREEN->line_length = col - 1;

        FB_UNLOCK();
    }

    if( current_pos )
        *current_pos = tmp_current_pos;

	return res;
}
